package com.lantone.qc.kernel.catalogue.yiwu.firstpagerecord;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.kernel.util.RegularUtil;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @ClassName : FIRP0161
 * @Description : 患者年龄与入院记录不一致
 * @Author : 胡敬
 * @Date: 2020-03-16 10:00
 */
@Component
public class FIRP0161 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        if (inputInfo.getFirstPageRecordDoc() != null && inputInfo.getFirstPageRecordDoc().getStructureMap() != null
                && inputInfo.getBeHospitalizedDoc() != null && inputInfo.getBeHospitalizedDoc().getStructureMap() != null) {
            Map<String, String> firstpageStructureMap = inputInfo.getFirstPageRecordDoc().getStructureMap();
            Map<String, String> beHospitalStructureMap = inputInfo.getBeHospitalizedDoc().getStructureMap();
            String firstAdmissionAge = firstpageStructureMap.get(Content.age);
            /* 根据病案首页新生儿出生月数和新生儿出生天数拼接年龄 */
            String newbornAgeMonths = firstpageStructureMap.get(Content.newbornAgeMonths);
            String newbornAgeDays = firstpageStructureMap.get(Content.newbornAgeDays);
            String newbornAgeStr = "";
            int newbornAge = 0;
            if (StringUtil.isNotBlank(newbornAgeMonths) && StringUtil.isNotBlank(newbornAgeDays)) {
                newbornAgeStr = newbornAgeMonths + newbornAgeDays;
            }
            if (StringUtil.isNotBlank(newbornAgeStr)) {
                newbornAge = removalUnit(newbornAgeStr);
            }
            String admissionAge = beHospitalStructureMap.get(Content.age);
            if (StringUtil.isBlank(firstAdmissionAge) || StringUtil.isBlank(admissionAge)) {
                status.set("0");
                return;
            }
//            String admissAge = "";
            if (StringUtil.isNotBlank(admissionAge) && admissionAge.contains("岁")) {
                admissionAge = admissionAge.substring(0, admissionAge.indexOf("岁"));
            }
            if (StringUtil.isNotBlank(admissionAge) && admissionAge.contains("天")) {
                admissionAge = admissionAge.substring(0, admissionAge.indexOf("天"));
            }
            if (StringUtil.isNotBlank(admissionAge) && admissionAge.contains("分")) {
                admissionAge = admissionAge.substring(0, admissionAge.indexOf("分"));
            }
            if (StringUtil.isNotBlank(admissionAge) && (admissionAge.contains("时") || admissionAge.contains("小时"))) {
                admissionAge = admissionAge.contains("小时") ? admissionAge.substring(0, admissionAge.indexOf("小")) : admissionAge.substring(0, admissionAge.indexOf("时"));
            }
            if (StringUtil.isNotBlank(firstAdmissionAge) && firstAdmissionAge.contains("岁")) {
                firstAdmissionAge = firstAdmissionAge.substring(0, firstAdmissionAge.indexOf("岁"));
            }
            if (StringUtil.isNotBlank(firstAdmissionAge) && firstAdmissionAge.contains("天")) {
                firstAdmissionAge = firstAdmissionAge.substring(0, firstAdmissionAge.indexOf("天"));
            }
            if (StringUtil.isNotBlank(firstAdmissionAge) && (firstAdmissionAge.contains("小时") || firstAdmissionAge.contains("时"))) {
                firstAdmissionAge = firstAdmissionAge.contains("小时") ? firstAdmissionAge.substring(0, firstAdmissionAge.indexOf("小")) : firstAdmissionAge.substring(0, firstAdmissionAge.indexOf("时"));
            }
            if (StringUtil.isNotBlank(firstAdmissionAge) && firstAdmissionAge.contains("分")) {
                firstAdmissionAge = firstAdmissionAge.substring(0, firstAdmissionAge.indexOf("分"));
            }
            // int firstAdmissionAgeNum = removalUnit(firstAdmissionAge);
            // int admissionAgeNum = removalUnit(admissionAge);
            // 只比较年龄前的数值
            if (!admissionAge.equals(firstAdmissionAge)) {
                status.set("-1");
                Map<String, Object> structureExtMap = inputInfo.getFirstPageRecordDoc().getStructureExtMap();
                String xmlId = CatalogueUtil.getXmlId(structureExtMap, Content.age);
                id.set(xmlId);
                return;
            }
            // int firstAdmissionAgeNum = removalUnit(firstAdmissionAge);
            // int admissionAgeNum = removalUnit(admissionAge);
            //
            // if (firstAdmissionAgeNum != admissionAgeNum && newbornAge != admissionAgeNum) {
            //     status.set("-1");
            // }
        }
    }

    private int removalUnit(String admissionAge) {
        int age = 0;
        if (StringUtil.isNotBlank(admissionAge) && admissionAge.contains("岁")) {
            admissionAge = admissionAge.replaceAll("[岁天]", "");
        }
        // 解决邵逸夫数据，年龄后跟着身份证号码
        admissionAge = RegularUtil.ClearBracket(admissionAge);
        admissionAge = admissionAge.replaceAll("[^0-9]", "");
        if (CatalogueUtil.numbersOnly(admissionAge)) {
            age = Integer.parseInt(admissionAge);
        }
        return age;
    }
}