package com.lantone.qc.trans.ywzxyy;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.pub.model.doc.FirstCourseRecordDoc;
import com.lantone.qc.pub.model.vo.MedrecVo;
import com.lantone.qc.pub.util.FastJsonUtils;
import com.lantone.qc.trans.ModelDocTrans;
import com.lantone.qc.trans.comsis.ModelDocGenerate;
import com.lantone.qc.trans.comsis.OrdinaryAssistant;
import com.lantone.qc.trans.ywzxyy.util.YwzxyyFirstCourseRecordXmlAnalysis;
import com.lantone.qc.trans.ywzxyy.util.YwzxyyXmlAnalysis;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;

import java.util.List;
import java.util.Map;

import static com.lantone.qc.trans.comsis.ModelDocGenerate.structureMapJoin;

/**
 * @Description: 首次病程录文档生成
 * @author: rengb
 * @time: 2020/3/17 13:25
 */
@Slf4j
public class YwzxyyFirstCourseRecordDocTrans extends ModelDocTrans {

    @Override
    public FirstCourseRecordDoc extract(MedrecVo medrecVo) {
        FirstCourseRecordDoc firstCourseRecordDoc = null;
        try {
            Map<String, Object> contentMap = ((List<Map>) medrecVo.getContent().get("content")).get(0);
            String content = contentMap.get("xmlText").toString();
            Map<String, String> structureMap = null;
            if (contentMap.get("isParsed") != null && "1".equals(contentMap.get("isParsed").toString())) {
                structureMap = (Map) FastJsonUtils.getJsonToMap(content);
            } else {
                String recTitle = contentMap.get("recTitle").toString();
                String recTypeId = contentMap.get("recTypeId").toString();
                YwzxyyXmlAnalysis ywzxyyXmlAnalysis = new YwzxyyFirstCourseRecordXmlAnalysis();
                Map<String, String> sourceMap = ywzxyyXmlAnalysis.analysis(content, recTitle, recTypeId);
                if (MapUtils.isNotEmpty(sourceMap)) {
                    sourceMap = removeSerialNumber(sourceMap);
                    structureMap = OrdinaryAssistant.mapKeyContrast(sourceMap, keyContrasts, "2");
                    structureMap.put("记录编号", contentMap.get("recId").toString());
                    structureMap.put("病历号", contentMap.get("behospitalCode") == null ? null : contentMap.get("behospitalCode").toString());
                }
            }
            if (MapUtils.isNotEmpty(structureMap)) {
                firstCourseRecordDoc = ModelDocGenerate.firstCourseRecordDocGen(structureMap);
                firstCourseRecordDoc.setText(structureMap.get("原始文本"));
                firstCourseRecordDoc.setPageData((Map) structureMap);
                List<String> keys = Lists.newArrayList("诊疗计划");
                String treatPlanJoin = structureMapJoin(structureMap, keys);
                firstCourseRecordDoc.getTreatPlanLabel().setAiText(treatPlanJoin);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return firstCourseRecordDoc;
    }

    private Map<String, String> removeSerialNumber(Map<String, String> sourceMap) {
        Map<String, String> structureMap = Maps.newLinkedHashMap();
        sourceMap.forEach((key, value) -> structureMap.put(key.replaceAll("[一二三四五六()（）.123456]", ""), value));
        return structureMap;
    }

    private List<String> keyContrasts = Lists.newArrayList(
            "病历记录时间=记录时间"
    );

}